C     LIBRARY GRST
C
C-----
C     The module GRST are a set of routines to compute
C     mutual visibility of a satellite and one or more
C     ground stations.
C
C-----
C
      SUBROUTINE STSTST (GSFN, NAME, IGST, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine STSTST stores data of a selected ground
C     station in a sequential common.
C
C INPUTS:
C     GSFN  Ground station file name.
C     NAME  Ground station's name (maximum 18 characters).
C
C OUTPUTS:
C     IGST  Ground station identification number
C     IFLG  Output flag:
C           0 - Normal operation.
C           990 - Ground station file not found.
C           991 - Ground station name does not match.
C           992 - It was tried to store more than 20 ground
C                 stations in the station's common.
C           OTHER Number different from zero - an input
C                 error occurs.
C
C SUBCALLS:
C     STGRST
C
C OBS:
C     1 - The default ground station file is 'GRSTA.DAT'
C         (unit=11).
C     2 - The selected ground station (NAME) is stored 
C         sequentialy in the common GRSTN, GRSCT and GRSCS,
C         in the subroutine STGRST.
C     3 - At each call in the STSTST subroutine, the number
C         of stored ground stations (NGST) is increased of
C         one unit.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION GS(4)
      CHARACTER GSFN*(*), NAME*(*)
      CHARACTER*18 NACO
      LOGICAL LEXT

C
C------
C
      INQUIRE (FILE=GSFN, EXIST=LEXT)
      IF (LEXT) THEN
        NACO   = '+-'
        IETS   = 0
        OPEN (UNIT=11, FILE=GSFN, READONLY,
     1        STATUS='OLD')
        DO WHILE (NACO.NE.NAME.AND.IETS.GE.0)
         READ (11,FMT=*,IOSTAT=IETS) NACO, GS
        ENDDO
        CLOSE (UNIT=11)
        IF (IETS.EQ.0) THEN
          CALL STGRST (NACO, GS, IGST, IFLG)
          IF (IFLG.NE.0) IFLG = IFLG + 991
        ELSE
          IFLG  = IETS
        ENDIF
      ELSE
        IFLG   = 990
      ENDIF

      RETURN
      END

      SUBROUTINE STGRST (NAME, GS, IGST, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine STGRST stores data of a given ground
C     station in a sequential common.
C
C INPUTS:
C     NAME  Ground station's name (maximum 18 characters).
C
C     GS(1) Ground station longitude (deg)
C     GS(2) Ground station latitude (deg)
C     GS(3) Ground station altitude (meters)
C     GS(4) Ground station antenna minimum elevation (deg)
C
C     COMMON /CONST/ PI, PIV2, PID2, RAD, DEG
C           PI    = 3.14159265...
C           PIV2  = 2*PI
C           PID2  = PI/2.
C           RAD   = PI/180.
C           DEG   = 180./PI
C
C     COMMON /EARTH/ RE, FLAT, GME, TETD
C           RE    = Earth radius (meters)
C           FLAT  = Earth flateness factor
C           GME   = Earth gravitational constant
C           TETD  = Earth rotational rate (rad/sec)
C
C OUTPUTS:
C     IGST  Ground station identification number
C     IFLG  Output flag:
C           0 - Normal operation
C           1 - It was tried to store more than 20 ground
C               stations in the output common.
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           stations can be handled by this routine.
C
C     COMMON /GRSCT/ RO, RA, HA, EV
C           RO(I) Longitude (rad) of the ground station
C           RA(I) Latitude (rad) of the ground station
C           HA(I) Altitude (meters) of the ground station
C           EV(I) minimum ground station antenna elevation
C                 (rad)
C
C     COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C           SO(I) SIN of the ground station longitude
C           CO(I) COS of the ground station longitude
C           SA(I) SIN of the ground station latitude
C           CA(I) COS of the ground station latitude
C           XS(I) GS position (meters) in the equator plane
C           ZS(I) GS position in the Earth's rotation axis
C           TE(I) TANGENT of the GS minimum elevation
C
C OBS:
C     1 - The given ground station (NAME and data) is stored 
C         sequentialy in the common GRSTN, GRSCT, GRSCS
C     2 - At each call in the STGRST subroutine, the number
C         of stored ground stations (NGST) is increased of
C         one unit.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION GS(4)
      DIMENSION RO(20), RA(20), HA(20), EV(20)
      DIMENSION CO(20), SO(20), CA(20), SA(20)
      DIMENSION XS(20), ZS(20), TE(20)
      CHARACTER    NAME*(*)
      CHARACTER*18 GRNA(20), NACO

      COMMON /CONST/ PI, PIV2, PID2, RAD, DEG
      COMMON /EARTH/ RE, FLAT, GME, TETD

      COMMON /GRSTN/ NGST, GRNA
      COMMON /GRSCT/ RO, RA, HA, EV
      COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C
C------
C
      IF (NGST.GE.20) THEN
        IFLG  = 1
        RETURN
      ENDIF

      IFLG    = 0
      NGST    = NGST + 1
      GRNA(NGST) = NAME

      UMF     = 1.D0 - FLAT
      UM2     = UMF*UMF
      DMF     = FLAT*(1.D0 + UMF)

      RLON    = GS(1)*RAD
      RLAT    = GS(2)*RAD
      RO(NGST)= RLON
      RA(NGST)= RLAT
      HA(NGST)= GS(3)
      EV(NGST)= GS(4)*RAD
      SO(NGST)= SIN(RLON)
      CO(NGST)= COS(RLON)
      SIRL    = SIN(RLAT)
      SA(NGST)= SIRL
      CA(NGST)= COS(RLAT)
      GEOI    = RE/SQRT(1.D0 - DMF*SIRL*SIRL)
      XS(NGST)= (GEOI + GS(3))*CA(NGST)
      ZS(NGST)= (GEOI*UM2 + GS(3))*SIRL
      TE(NGST)= TAN(EV(NGST))
      IGST    = NGST
      RETURN
      END


      SUBROUTINE GRSTVS (IGST, XT, XP, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine GRSTVS verifies if a given position
C     is visible from a ground station.
C
C INPUTS:
C     IGST  Ground station identification number.
C
C     XT(3) Components of the satellite position in
C           terrestrial geocentric coordinates (meters).
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           station can be handled by this routine.
C
C     COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C           SO(I) SIN of the ground station longitude
C           CO(I) COS of the ground station longitude
C           SA(I) SIN of the ground station latitude
C           CA(I) COS of the ground station latitude
C           XS(I) GS position (meters) in the equator plane
C           ZS(I) GS position in the Earth's rotation axis
C           TE(I) TANGENT of the GS minimum elevation
C
C OUTPUTS:
C     XP(3) Components of the satellite position in
C           topocentric coordinates (Z-zenith, Y-east),
C           in meters.
C
C     IFLG  Output flag:
C           0 - Satellite position is visible from the
C               ground station.
C           1 - Satellite is not visible.
C           -1- Ground station not defined.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C     Valder Matos de Medeiros
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION XT(3), XP(3)
      DIMENSION CO(20), SO(20), CA(20), SA(20)
      DIMENSION XS(20), ZS(20), TE(20)
      CHARACTER*18 GRNA(20)

      COMMON /GRSTN/ NGST, GRNA
      COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C
C------
C
      IF (IGST.LE.0.OR.IGST.GT.NGST) THEN
       IFLG = -1
       RETURN
      ENDIF

      XSAT  = XT(1)
      YSAT  = XT(2)
      ZSAT  = XT(3)

      XLME  = XSAT*CO(IGST) + YSAT*SO(IGST) - XS(IGST)
      YLME  = YSAT*CO(IGST) - XSAT*SO(IGST)
      ZLME  = ZSAT - ZS(IGST)

      XTOP  = XLME*SA(IGST) - ZLME*CA(IGST)
      YTOP  = YLME
      ZTOP  = XLME*CA(IGST) + ZLME*SA(IGST)

      TELV  = ZTOP/SQRT(XTOP*XTOP + YTOP*YTOP)

      XP(1) = XTOP
      XP(2) = YTOP
      XP(3) = ZTOP

      IF (TELV.GE.TE(IGST)) THEN
       IFLG = 0
      ELSE
       IFLG = 1
      ENDIF

      RETURN
      END


      SUBROUTINE GRSSVS (XT, IV)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine GRSSVS verifies if a given position
C     is visible from all the secected ground stations.
C
C INPUTS:
C     XT(3) Components of the satellite position in
C           terrestrial geocentric coordinates (meters).
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           station can be handled by this routine.
C
C OUTPUTS:
C     IV(20)  Output flag for each ground station:
C             0 - Satellite position is visible from the
C                 ground station.
C             1 - Satellite is not visible.
C
C SUBCALL:
C     GRSTVS
C
C AUTHOR:
C     Valdemir Carrara       MAR 1990        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION XT(3), XP(3)
      DIMENSION IV(20)
      CHARACTER*18 GRNA(20)

      COMMON /GRSTN/ NGST, GRNA
C
C------
C

      DO IG = 1, NGST

        CALL GRSTVS (IG, XT, XP, IFLG)
        IV(IG) = IFLG

      ENDDO

      RETURN
      END


      SUBROUTINE GRSTES (IGST, XT, XP, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine GRSTES detects the entrance and
C     exit of a satellite in visibility of a given
C     ground station.
C
C INPUTS:
C     IGST  Ground station identification number.
C
C     XT(3) Components of the satellite position in
C           terrestrial geocentric coordinates (meters).
C
C     COMMON /GRVIS/ IP(20)
C           The array IP stores the status of the
C           previous position relative to the ground
C           stations:
C           0 - Satellite position was visible from the
C               ground station.
C           1 - Satellite was not visible.
C
C OUTPUTS:
C     XP(3) Components of the satellite position in
C           topocentric coordinates (X-zenith, Y-east),
C           in meters.
C
C     IFLG  Output flag:
C           0 - Condition is the same as before.
C           1 - The satellite entered in visibility.
C           -1- The satellite exits from visibility.
C
C SUBCALL:
C     GRSTVS
C
C AUTHOR:
C     Valdemir Carrara       MAR 1990        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION XT(3), XP(3)
      DIMENSION IP(20)

      COMMON /GRVIS/ IP
C
C------
C

      CALL GRSTVS (IGST, XT, XP, IFAC)

      IF (IFAC.EQ.0) THEN
        IF (IP(IGST).NE.0) THEN
          IFLG = 1
          IP(IGST) = 0
        ELSE
          IFLG = 0
        ENDIF
      ELSE
        IF (IFAC.EQ.1) THEN
          IF (IP(IGST).EQ.0) THEN
            IFLG = -1
            IP(IGST) = 1
          ELSE
            IFLG = 0
          ENDIF
        ELSE
          IFLG = IFAC
        ENDIF
      ENDIF

      RETURN
      END


      SUBROUTINE GRSTEI (IGST, XT, TIME, XP, IFLG,
     1                   XO, TOUT, ICON)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine GRSTEI detects the entrance and
C     exit of a satellite in visibility of a given
C     ground station, and calculates the intersection
C     point.
C
C INPUTS:
C     IGST  Ground station identification number.
C
C     XT(3) Components of the satellite position in
C           terrestrial geocentric coordinates (meters).
C
C     TIME  Time associated to the satellite position.
C           it must be continuous.
C
C     COMMON /GRVIS/ IP(20)
C           The array IP stores the status of the
C           previous position relative to the ground
C           stations:
C           0 - Satellite position was visible from the
C               ground station.
C           1 - Satellite was not visible.
C
C     COMMON /GRPOS/ XA(20), YA(20), ZA(20), TI(20)
C           This commons stores the last position
C           of the satellite and the time, in topocentric
C           coordinates.
C
C     COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C           SO(I) SIN of the ground station longitude
C           CO(I) COS of the ground station longitude
C           SA(I) SIN of the ground station latitude
C           CA(I) COS of the ground station latitude
C           XS(I) GS position (meters) in the equator plane
C           ZS(I) GS position in the Earth's rotation axis
C           TE(I) TANGENT of the GS minimum elevation
C
C OUTPUTS:
C     XP(3) Components of the satellite position in
C           topocentric coordinates (Z-zenith, Y-east),
C           in meters.
C
C     IFLG  Output flag:
C           0 - Condition is the same as before.
C           1 - The satellite entered in visibility.
C           -1- The satellite exits from visibility.
C
C     XO    Components of the satellite position in
C           topocentric coordinates interpolated to
C           the instant of the entrance/exit from
C           visibility.
C
C     TOUT  Interpolated time of the satellite entrance
C           or exit from visibility.
C
C     ICON  Output condition flag:
C           -1- The satellite did not enter or exit the
C               visibility.
C           0 - The results stored in XO and TOUT are valid.
C           Other value: The results are not valid.
C
C SUBCALL:
C     GRSTVS
C     GRSINT
C
C AUTHOR:
C     Valdemir Carrara       MAR 1990        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION XT(3), XP(3), XO(3)
      DIMENSION XA(20), YA(20), ZA(20), TI(20)
      DIMENSION XN(4), XV(4), XI(4)
      DIMENSION CO(20), SO(20), CA(20), SA(20)
      DIMENSION XS(20), ZS(20), TE(20)
      DIMENSION IP(20)

      COMMON /GRVIS/ IP
      COMMON /GRPOS/ XA, YA, ZA, TI
      COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C
C------
C

      CALL GRSTVS (IGST, XT, XP, IFAC)

      IF (IFAC.EQ.0) THEN
        IF (IP(IGST).NE.0) THEN
          IFLG = 1
          IP(IGST) = 0
          XN(1)= XP(1)
          XN(2)= XP(2)
          XN(3)= XP(3)
          XV(1)= XA(IGST)
          XV(2)= YA(IGST)
          XV(3)= ZA(IGST)
          TOLD = TI(IGST)
          CELE = TE(IGST)
          CALL GRSINT (XN, TIME, XV, TOLD, CELE, XO, TOUT, ICON)
        ELSE
          IFLG = 0
          ICON = -1
        ENDIF
        XA(IGST) = XP(1)
        YA(IGST) = XP(2)
        ZA(IGST) = XP(3)
        TI(IGST) = TIME
      ELSE
        IF (IFAC.EQ.1) THEN
          IF (IP(IGST).EQ.0) THEN
            IFLG = -1
            IP(IGST) = 1
            XN(1)= XP(1)
            XN(2)= XP(2)
            XN(3)= XP(3)
            XV(1)= XA(IGST)
            XV(2)= YA(IGST)
            XV(3)= ZA(IGST)
            TOLD = TI(IGST)
            CELE = TE(IGST)
            CALL GRSINT (XN, TIME, XV, TOLD, CELE, XO,
     1                  TOUT, ICON)
          ELSE
            IFLG = 0
          ENDIF
          XA(IGST) = XP(1)
          YA(IGST) = XP(2)
          ZA(IGST) = XP(3)
          TI(IGST) = TIME
        ELSE
          IFLG = IFAC
        ENDIF
      ENDIF

      RETURN
      END

      SUBROUTINE GRSINT (XN, TIME, XV, TOLD, CELE, XO,
     1                  TOUT, ICON)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     To compute the instant of entrance/exit in visibility
C     of a satellite.
C
C INPUTS:
C     XN    Satellite position at instant TIME, in topocen-
C           tric coordinates.
C     TIME  Instant associated to the satellite position
C     XV    Last satellite position, in topocentric coordi-
C           nates.
C     TOLD  Instant associated to the last satellite posi-
C           tion.
C     CELE  TANGENT of the antenna minimun elevation.
C
C OUTPUTS:
C     XO    Interpolated topocentric coordinates of the
C           satellite at the entrance/exit from visibility.
C
C     TOUT  Instant associated to the interpolated elements.
C
C     ICON  Output flag condition:
C           0 - Results are valid.
C           Other values: Results are not valid.
C
C AUTHOR:
C     Valdemir Carrara   APR/90
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION XN(3), XV(3), XO(3)
C
C-----
C
      DELX                 = XN(1) - XV(1)
      DELY                 = XN(2) - XV(2)
      DELZ                 = XN(3) - XV(3)
      DETT                 = TIME - TOLD

      IF (CELE.NE.0.) THEN
        CEL2               = CELE*CELE
        IF (DELZ.NE.0.) THEN
          AX               = DELX/DELZ
          AY               = DELY/DELZ
          AT               = DETT/DELZ
          BX               = XV(1) - AX*XV(3)
          BY               = XV(2) - AY*XV(3)
          BT               = TOLD - AT*XV(3)
          AA               = CEL2*(AX*AX + AY*AY) - 1.
          BB               = 2.*CEL2*(AX*BX + AY*BY)
          CC               = CEL2*(BX*BX + BY*BY)
          IF (AA.NE.0.) THEN
            DELT           = BB*BB - 4.*AA*CC
            IF (DELT.GT.0.) THEN
              DELT         = SQRT(DELT)
              ZBA1         = (-BB - DELT)/2./AA
              ZBA2         = (-BB + DELT)/2./AA
              DEZ1         = (ZBA1 - XV(3))/DELZ
              DEZ2         = (ZBA2 - XV(3))/DELZ
              IF (DEZ1.GE.0.AND.DEZ1.LE.1.) THEN
                IF (DEZ2.GE.0.AND.DEZ2.LE.1.) THEN
                  ICON     = 2
                ELSE
                  XO(1)    = AX*ZBA1 + BX
                  XO(2)    = AY*ZBA1 + BY
                  XO(3)    = ZBA1
                  TOUT     = AT*ZBA1 + BT
                  ICON     = 0
                ENDIF
              ELSE
                IF (DEZ2.GE.0.AND.DEZ2.LE.1.) THEN
                  XO(1)    = AX*ZBA2 + BX
                  XO(2)    = AY*ZBA2 + BY
                  XO(3)    = ZBA2
                  TOUT     = AT*ZBA2 + BT
                  ICON     = 0
                ELSE
                  ICON     = 1
                ENDIF
              ENDIF
            ELSE
              IF (DELT.EQ.0.) THEN
                ZBAR       = -BB/2./AA
                DEZ1       = (ZBAR - XV(3))/DELZ
                IF (DEZ1.GE.0.AND.DEZ1.LE.1.) THEN
                  XO(1)    = AX*ZBAR + BX
                  XO(2)    = AY*ZBAR + BY
                  XO(3)    = ZBAR
                  TOUT     = AT*ZBAR + BT
                  ICON     = 0
                ELSE
                  ICON     = 3
                ENDIF
              ELSE
                ICON       = 4
              ENDIF
            ENDIF
          ELSE
            IF (BB.NE.0.) THEN
              ZBAR         = -CC/BB
              DEZ1         = (ZBAR - XV(3))/DELZ
              IF (DEZ1.GE.0.AND.DEZ1.LE.1.) THEN
                XO(1)      = AX*ZBAR + BX
                XO(2)      = AY*ZBAR + BY
                XO(3)      = ZBAR
                TOUT       = AT*ZBAR + BT
                ICON       = 0
              ELSE
                ICON       = 5
              ENDIF
            ENDIF
          ENDIF
        ELSE
          IF (DELX.NE.0.) THEN
            AY             = DELY/DELX
            AT             = DETT/DELX
            BY             = XV(2) - AY*XV(1)
            BT             = TOLD - AT*XV(1)
            AA             = CEL2*(1. + AY*AY)
            BB             = 2.*CEL2*AY*BY
            CC             = BY*BY - XV(3)*XV(3)
            DELT           = BB*BB - 4.*AA*CC
            IF (DELT.GE.0.) THEN
              DELT         = SQRT(DELT)
              IF (DELT.NE.0.) THEN
                XBA1       = (-BB - DELT)/2./AA
                XBA2       = (-BB - DELT)/2./AA
                DEX1       = (XBA1 - XV(1))/DELX
                DEX2       = (XBA2 - XV(1))/DELX
                IF (DEX1.GE.0.AND.DEX1.LE.1.) THEN
                  IF (DEX2.GE.0.AND.DEX2.LE.1.) THEN
                    ICON   = 8
                  ELSE
                    XO(1)  = XBA1
                    XO(2)  = AY*XBA1 + BY
                    XO(3)  = XV(3)
                    TOUT   = AT*XBA1 + BT
                    ICON   = 0
                  ENDIF
                ELSE
                  IF (DEX2.GE.0.AND.DEX2.LE.1.) THEN
                    XO(1)  = XBA2
                    XO(2)  = AY*XBA2 + BY
                    XO(3)  = XV(3)
                    TOUT   = AT*XBA2 + BT
                    ICON   = 0
                  ELSE
                    ICON   = 7
                  ENDIF
                ENDIF
              ELSE
                XBAR       = -BB/2./AA
                DEX1       = (XBAR - XV(1))/DELX
                IF (DEX1.GE.0.AND.DEX1.LE.1.) THEN
                  XO(1)    = XBAR
                  XO(2)    = AY*XBAR + BY
                  XO(3)    = XV(3)
                  TOUT     = AT*XBAR + BT
                  ICON     = 0
                ELSE
                  ICON     = 9
                ENDIF
              ENDIF
            ELSE
              ICON         = 10
            ENDIF
          ELSE
            IF (DELY.NE.0.) THEN
              AT           = DETT/DELY
              BT           = TOLD - AT*XV(2)
              DELT         = XV(3)*XV(3)/CEL2 - XV(1)*XV(1)
              IF (AA.GE.0.) THEN
                IF (AA.NE.0.) THEN
                  DELT     = SQRT(DELT)
                  YBA1     = -DELT
                  YBA2     =  DELT
                  DEY1     = (YBA1 - XV(2))/DELY
                  DEY2     = (YBA2 - XV(2))/DELY
                  IF (DEY1.GE.0.AND.DEY1.LE.1.) THEN
                    IF (DEY2.GE.0.AND.DEY2.LE.1.) THEN
                      ICON = 12
                    ELSE
                      XO(1)= XV(1)
                      XO(2)= YBA1
                      XO(3)= XV(3)
                      TOUT = AT*YBA1 + BT
                      ICON = 0
                    ENDIF
                  ELSE
                    IF (DEY2.GE.0.AND.DEY2.LE.1.) THEN
                      XO(1)= XV(1)
                      XO(2)= YBA2
                      XO(3)= XV(3)
                      TOUT = AT*YBA2 + BT
                      ICON = 0
                    ELSE
                      ICON = 11
                    ENDIF
                  ENDIF
                ELSE                  
                  YBA1     = 0.
                  DEY1     = (YBA1 - XV(2))/DELY
                  IF (DEY1.GE.0.AND.DEY1.LE.1.) THEN
                    XO(1)  = XV(1)
                    XO(2)  = YBA1
                    XO(3)  = XV(3)
                    TOUT   = AT*YBA1 + BT
                    ICON   = 0
                  ELSE
                    ICON   = 13
                  ENDIF
                ENDIF
              ELSE
                ICON       = 14
              ENDIF
            ELSE
              ICON         = 15
            ENDIF
          ENDIF
        ENDIF
      ELSE
        IF (DELZ.NE.0.) THEN
          XO(1)            = XV(1) - DELX/DELZ*XV(3)
          XO(2)            = XV(2) - DELY/DELZ*XV(3)
          XO(3)            = 0.
          TOUT             = TOLD - DETT/DELZ*XV(3)
          ICON             = 0
        ELSE
          ICON             = 16
        ENDIF
      ENDIF

      RETURN
      END


      SUBROUTINE GRSSES (XT, IV)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine GRSSES detects the entrance and
C     exit of a satellite in visibility from all the
C     selected ground stations.
C
C INPUTS:
C     XT(3) Components of the satellite position in
C           terrestrial geocentric coordinates (meters).
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           station can be handled by this routine.
C
C OUTPUTS:
C     IV(20)  Output flag:
C             0 - Condition is the same as before.
C             1 - The satellite entered in visibility.
C            -1 - The satellite exits from visibility.
C
C SUBCALL:
C     GRSTES
C
C AUTHOR:
C     Valdemir Carrara       MAR 1990        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION XT(3), XP(3)
      DIMENSION IV(20)
      CHARACTER*18 GRNA(20)

      COMMON /GRSTN/ NGST, GRNA
C
C------
C

      DO IG = 1, NGST

        CALL GRSTES (IG, XT, XP, IFLG)
        IV(IG) = IFLG

      ENDDO

      RETURN
      END


      SUBROUTINE INQGSI (NAME, IGST, GS, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine INQGSI returns with the identification
C     number of a ground station previously stored in the
C     ground station common.
C
C INPUTS:
C     NAME  Ground station's name (maximum 18 characters).
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           station can be handled by this routine.
C
C     COMMON /GRSCT/ RO, RA, HA, EV
C           RO(I) Longitude (rad) of the ground station
C           RA(I) Latitude (rad) of the ground station
C           HA(I) Altitude (meters) of the ground station
C           EV(I) Minimum ground station antenna elevation
C                 (rad)
C
C OUTPUTS:
C     IGST  Number of the selected ground station.
C
C     GS(1) Ground station longitude (rad)
C     GS(2) Ground station latitude (rad)
C     GS(3) Ground station altitude (meters)
C     GS(4) Ground station antenna minimum elevation (rad)
C
C     IFLG  Output flag:
C           0 - Normal operation.
C           Other number different from zero - Ground
C           station not found.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION GS(4)
      DIMENSION RO(20), RA(20), HA(20), EV(20)
      CHARACTER    NAME*(*)
      CHARACTER*18 GRNA(20), NACO

      COMMON /GRSTN/ NGST, GRNA
      COMMON /GRSCT/ RO, RA, HA, EV
C
C------
C
      IGST   = 1
      IFLG   = 0

      DO WHILE (NAME.NE.GRNA(IGST).AND.IGST.LE.NGST)
       IGST  = IGST + 1
      ENDDO

      IF (IGST.GT.NGST) THEN
       IFLG  = 1
       RETURN
      ELSE
       GS(1) = RO(IGST)
       GS(2) = RA(IGST)
       GS(3) = HA(IGST)
       GS(4) = EV(IGST)
      ENDIF

      RETURN
      END


      SUBROUTINE INQGSD (IGST, NAME, GS, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine INQGSD returns with the NAME and data
C     of a ground station given by its identification
C     number.
C
C INPUTS:
C     IGST  Number of the selected ground station.
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           station can be handled by this routine.
C
C     COMMON /GRSCT/ RO, RA, HA, EV
C           RO(I) Longitude (rad) of the ground station
C           RA(I) Latitude (rad) of the ground station
C           HA(I) Altitude (meters) of the ground station
C           EV(I) Minimum ground station antenna elevation
C                 (rad)
C
C OUTPUTS:
C     NAME  Ground station's name (maximum 18 characters).
C
C     GS(1) Ground station longitude (rad)
C     GS(2) Ground station latitude (rad)
C     GS(3) Ground station altitude (meters)
C     GS(4) Ground station antenna minimum elevation (rad)
C
C     IFLG  Output flag:
C           0 - Normal operation.
C           1 - Identification number inexistent
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION GS(4)
      DIMENSION RO(20), RA(20), HA(20), EV(20)
      CHARACTER    NAME*(*)
      CHARACTER*18 GRNA(20), NACO

      COMMON /GRSTN/ NGST, GRNA
      COMMON /GRSCT/ RO, RA, HA, EV
C
C------
C
      IF (IGST.GE.1.AND.IGST.LE.NGST) THEN
       IFLG  = 0
       NAME  = GRNA(IGST)
       GS(1) = RO(IGST)
       GS(2) = RA(IGST)
       GS(3) = HA(IGST)
       GS(4) = EV(IGST)
      ELSE
       IFLG  = 1
      ENDIF

      RETURN
      END


      SUBROUTINE INQGSN (NTGS)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine INQGSN returns with the total number of
C     ground stations previously stored in the common.
C
C INPUTS:
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           station can be handled by this routine.
C
C OUTPUTS:
C     NTGS  Total number of selected ground stations.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      CHARACTER*18 GRNA(20)

      COMMON /GRSTN/ NGST, GRNA
C
C------
C
      NTGS   = NGST

      RETURN
      END


      SUBROUTINE CLEGSC
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine CLEGSC initializes the ground station
C     commons, by setting the total number of selected
C     stations equal to zero.
C
C INPUTS:
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name (not used).
C
C OUTPUTS:
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations (equal
C                 to zero).
C           GRNA  Ground station's name.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)
      CHARACTER*18 GRNA(20)

      COMMON /GRSTN/ NGST, GRNA
C
C------
C
      NGST   = 0

      RETURN
      END


      SUBROUTINE REMGST (IGST, IFLG)
C
C-----
C
C MODULE: Orbit Graphics
C
C PURPOSE:
C     The subroutine REMGST removes a given ground station
C     from the ground station's common.
C
C INPUTS:
C     IGST  Ground station identification number, to be
C           deleted.
C
C     COMMON /GRSTN/ NGST, GRNA*18(20)
C           NGST  Number of stored ground stations.
C           GRNA  Ground station's name.
C           A maximum of 20 (18 characters each) ground
C           stations can be handled by this routine.
C
C     COMMON /GRSCT/ RO, RA, HA, EV
C           RO(I) Longitude (rad) of the ground station
C           RA(I) Latitude (rad) of the ground station
C           HA(I) Altitude (meters) of the ground station
C           EV(I) Minimum ground station antenna elevation
C                 (rad)
C
C     COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE
C           SO(I) SIN of the ground station longitude
C           CO(I) COS of the ground station longitude
C           SA(I) SIN of the ground station latitude
C           CA(I) COS of the ground station latitude
C           XS(I) GS position (meters) in the equator plane
C           ZS(I) GS position in the Earth's rotation axis
C           TE(I) TANGENT of the GS minimum elevation
C
C OUTPUTS:
C     IFLG  Output flag:
C           0 - Normal operation
C           1 - Ground station identification number out of
C               permitted range.
C OBS:
C     1 - The identification number of each stored ground
C         station is modified after the delection.
C
C AUTHOR:
C     Valdemir Carrara       JUN 1989        V 1.0
C
      IMPLICIT REAL*8 (A-H, O-Z)

      DIMENSION RO(20), RA(20), HA(20), EV(20)
      DIMENSION CO(20), SO(20), CA(20), SA(20)
      DIMENSION XS(20), ZS(20), TE(20)
      CHARACTER*18 GRNA(20)

      COMMON /GRSTN/ NGST, GRNA
      COMMON /GRSCT/ RO, RA, HA, EV
      COMMON /GRSCS/ SO, CO, SA, CA, XS, ZS, TE

C
C------
C
      IF (IGST.LE.0.OR.IGST.GT.NGST) THEN
        IFLG  = 1
        RETURN
      ELSE
        NGST  = NGST - 1
        DO II = IGST, NGST
          JJ       = II + 1
          GRNA(II) = GRNA(JJ)
          RO(II)   = RO(JJ)
          RA(II)   = RA(JJ)
          HA(II)   = HA(JJ)
          EV(II)   = EV(JJ)
          SO(II)   = SO(JJ)
          SA(II)   = SA(JJ)
          CO(II)   = CO(JJ)
          CA(II)   = CA(JJ)
          XS(II)   = XS(JJ)
          ZS(II)   = ZS(JJ)
          TE(II)   = TE(JJ)
        ENDDO
      ENDIF

      RETURN
      END
